@@ -9,6 +9,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <glib.h>
+#include <math.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/ioctl.h>
@@ -67,7 +68,7 @@ static struct device_info devices[MP_MAX_CAMERAS];
 static size_t num_devices = 0;
 
 static const struct mp_camera_config *camera = NULL;
-static MPCameraMode mode;
+static MPMode mode;
 
 static bool just_switched_mode = false;
 static int blank_frame_count = 0;
@@ -80,8 +81,6 @@ static int preview_height;
 
 static int device_rotation;
 
-static bool save_dng;
-
 struct control_state {
         bool gain_is_manual;
         int gain;
@@ -100,6 +99,33 @@ static bool want_focus = false;
 static MPPipeline *pipeline;
 static GSource *capture_source;
 
+static void
+mp_setup_media_link_pad_formats(struct device_info *dev_info,
+                                const struct mp_media_link_config media_links[],
+                                int num_media_links,
+                                MPMode *mode)
+{
+        const struct media_v2_entity *entities[2];
+        int ports[2];
+        for (int i = 0; i < num_media_links; i++) {
+                entities[0] = mp_device_find_entity(
+                        dev_info->device, (const char *)media_links[i].source_name);
+                entities[1] = mp_device_find_entity(
+                        dev_info->device, (const char *)media_links[i].target_name);
+                ports[0] = media_links[i].source_port;
+                ports[1] = media_links[i].target_port;
+
+                for (int j = 0; j < 2; j++)
+                        if (!mp_entity_pad_set_format(
+                                    dev_info->device, entities[j], ports[j], mode)) {
+                                g_printerr("Failed to set %s:%d format\n",
+                                           entities[j]->name,
+                                           ports[j]);
+                                exit(EXIT_FAILURE);
+                        }
+        }
+}
+
 static void
 setup_camera(MPDeviceList **device_list, const struct mp_camera_config *config)
 {
@@ -203,7 +229,12 @@ setup_camera(MPDeviceList **device_list, const struct mp_camera_config *config)
                 // Start with the capture format, this works around a bug with
                 // the ov5640 driver where it won't allow setting the preview
                 // format initially.
-                MPCameraMode mode = config->capture_mode;
+                MPMode mode = config->capture_mode;
+                if (config->num_media_links)
+                        mp_setup_media_link_pad_formats(dev_info,
+                                                        config->media_links,
+                                                        config->num_media_links,
+                                                        &mode);
                 mp_camera_set_mode(info->camera, &mode);
 
                 // Trigger continuous auto focus if the sensor supports it
@@ -311,7 +342,6 @@ update_process_pipeline()
                 .camera = camera,
                 .mode = mode,
                 .burst_length = burst_length,
-                .save_dng = save_dng,
                 .preview_width = preview_width,
                 .preview_height = preview_height,
                 .device_rotation = device_rotation,
@@ -322,6 +352,7 @@ update_process_pipeline()
                 .exposure = current_controls.exposure,
                 .has_auto_focus_continuous = info->has_auto_focus_continuous,
                 .has_auto_focus_start = info->has_auto_focus_start,
+                .flash_enabled = flash_enabled,
         };
         mp_process_pipeline_update_state(&pipeline_state);
 }
@@ -342,19 +373,34 @@ static void
 capture(MPPipeline *pipeline, const void *data)
 {
         struct camera_info *info = &cameras[camera->index];
-
-        captures_remaining = burst_length;
+        struct device_info *dev_info = &devices[info->device_index];
+        uint32_t gain;
+        float gain_norm;
 
         // Disable the autogain/exposure while taking the burst
         mp_camera_control_set_int32(info->camera, V4L2_CID_AUTOGAIN, 0);
         mp_camera_control_set_int32(
                 info->camera, V4L2_CID_EXPOSURE_AUTO, V4L2_EXPOSURE_MANUAL);
 
+        // Get current gain to calculate a burst length;
+        // with low gain there's 2, with the max automatic gain of the ov5640
+        // the value seems to be 248 which creates a 5 frame burst
+        // for manual gain you can go up to 11 frames
+        gain = mp_camera_control_get_int32(info->camera, V4L2_CID_GAIN);
+        gain_norm = (float)gain / (float)info->gain_max;
+        burst_length = (int)fmax(sqrt(gain_norm) * 10, 1) + 1;
+        captures_remaining = burst_length;
+
         // Change camera mode for capturing
         mp_process_pipeline_sync();
         mp_camera_stop_capture(info->camera);
 
         mode = camera->capture_mode;
+        if (camera->num_media_links)
+                mp_setup_media_link_pad_formats(dev_info,
+                                                camera->media_links,
+                                                camera->num_media_links,
+                                                &mode);
         mp_camera_set_mode(info->camera, &mode);
         just_switched_mode = true;
 
@@ -498,6 +544,7 @@ on_frame(MPBuffer buffer, void *_data)
 
                 if (captures_remaining == 0) {
                         struct camera_info *info = &cameras[camera->index];
+                        struct device_info *dev_info = &devices[info->device_index];
 
                         // Restore the auto exposure and gain if needed
                         if (!current_controls.exposure_is_manual) {
@@ -517,13 +564,19 @@ on_frame(MPBuffer buffer, void *_data)
                         mp_camera_stop_capture(info->camera);
 
                         mode = camera->preview_mode;
+                        if (camera->num_media_links)
+                                mp_setup_media_link_pad_formats(
+                                        dev_info,
+                                        camera->media_links,
+                                        camera->num_media_links,
+                                        &mode);
                         mp_camera_set_mode(info->camera, &mode);
                         just_switched_mode = true;
 
                         mp_camera_start_capture(info->camera);
 
                         // Disable flash
-                        if (info->flash) {
+                        if (info->flash && flash_enabled) {
                                 mp_flash_disable(info->flash);
                         }
 
@@ -532,6 +585,25 @@ on_frame(MPBuffer buffer, void *_data)
         }
 }
 
+static void
+mp_setup_media_link(struct device_info *dev_info,
+                    const struct mp_media_link_config *cfg,
+                    bool enable)
+{
+        const struct media_v2_entity *source_entity =
+                mp_device_find_entity(dev_info->device, cfg->source_name);
+
+        const struct media_v2_entity *target_entity =
+                mp_device_find_entity(dev_info->device, cfg->target_name);
+
+        mp_device_setup_entity_link(dev_info->device,
+                                    source_entity->id,
+                                    target_entity->id,
+                                    cfg->source_port,
+                                    cfg->target_port,
+                                    enable);
+}
+
 static void
 update_state(MPPipeline *pipeline, const struct mp_io_pipeline_state *state)
 {
@@ -552,6 +624,11 @@ update_state(MPPipeline *pipeline, const struct mp_io_pipeline_state *state)
                                              info->pad_id,
                                              dev_info->interface_pad_id,
                                              false);
+
+                        // Disable media links
+                        for (int i = 0; i < camera->num_media_links; i++)
+                                mp_setup_media_link(
+                                        dev_info, &camera->media_links[i], false);
                 }
 
                 if (capture_source) {
@@ -570,7 +647,18 @@ update_state(MPPipeline *pipeline, const struct mp_io_pipeline_state *state)
                                              dev_info->interface_pad_id,
                                              true);
 
+                        // Enable media links
+                        for (int i = 0; i < camera->num_media_links; i++)
+                                mp_setup_media_link(
+                                        dev_info, &camera->media_links[i], true);
+
                         mode = camera->preview_mode;
+                        if (camera->num_media_links)
+                                mp_setup_media_link_pad_formats(
+                                        dev_info,
+                                        camera->media_links,
+                                        camera->num_media_links,
+                                        &mode);
                         mp_camera_set_mode(info->camera, &mode);
 
                         mp_camera_start_capture(info->camera);
@@ -601,7 +689,6 @@ update_state(MPPipeline *pipeline, const struct mp_io_pipeline_state *state)
         preview_width = state->preview_width;
         preview_height = state->preview_height;
         device_rotation = state->device_rotation;
-        save_dng = state->save_dng;
 
         if (camera) {
                 struct control_state previous_desired = desired_controls;
