@@ -36,7 +36,7 @@ enum user_control { USER_CONTROL_ISO, USER_CONTROL_SHUTTER };
 
 static bool camera_is_initialized = false;
 static const struct mp_camera_config *camera = NULL;
-static MPCameraMode mode;
+static MPMode mode;
 
 static int preview_width = -1;
 static int preview_height = -1;
@@ -55,8 +55,6 @@ static bool has_auto_focus_start;
 
 static bool flash_enabled = false;
 
-static bool setting_save_dng;
-
 static MPProcessPipelineBuffer *current_preview_buffer = NULL;
 static int preview_buffer_width = -1;
 static int preview_buffer_height = -1;
@@ -65,7 +63,7 @@ static char last_path[260] = "";
 
 static MPZBarScanResult *zbar_result = NULL;
 
-static int burst_length = 3;
+static int burst_length = 0;
 
 // Widgets
 GtkWidget *preview;
@@ -110,7 +108,6 @@ update_io_pipeline()
                 .gain = gain,
                 .exposure_is_manual = exposure_is_manual,
                 .exposure = exposure,
-                .save_dng = setting_save_dng,
                 .flash_enabled = flash_enabled,
         };
         mp_io_pipeline_update_state(&io_state);
@@ -525,7 +522,8 @@ run_capture_action(GSimpleAction *action, GVariant *param, gpointer user_data)
 void
 run_about_action(GSimpleAction *action, GVariant *param, GApplication *app)
 {
-        gtk_show_about_dialog(NULL,
+        GtkWindow *parent = gtk_application_get_active_window(GTK_APPLICATION(app));
+        gtk_show_about_dialog(parent,
                               "program-name",
                               "Megapixels",
                               "title",
@@ -535,7 +533,7 @@ run_about_action(GSimpleAction *action, GVariant *param, GApplication *app)
                               "comments",
                               "The postmarketOS camera application",
                               "website",
-                              "https://sr.ht/~martijnbraam/megapixels",
+                              "https://gitlab.com/postmarketOS/megapixels",
                               "version",
                               VERSION,
                               "license-type",
@@ -700,14 +698,6 @@ static void
 run_close_settings_action(GSimpleAction *action, GVariant *param, gpointer user_data)
 {
         gtk_stack_set_visible_child_name(GTK_STACK(main_stack), "main");
-
-        // Update settings
-        bool save_dng = g_settings_get_boolean(settings, "save-raw");
-
-        if (save_dng != setting_save_dng) {
-                setting_save_dng = save_dng;
-                update_io_pipeline();
-        }
 }
 
 static void
@@ -1011,6 +1001,10 @@ activate(GtkApplication *app, gpointer data)
                 GTK_WIDGET(gtk_builder_get_object(builder, "flash-controls-button"));
         GtkWidget *setting_dng_button =
                 GTK_WIDGET(gtk_builder_get_object(builder, "setting-raw"));
+        GtkWidget *setting_postprocessor_combo =
+                GTK_WIDGET(gtk_builder_get_object(builder, "setting-processor"));
+        GtkListStore *setting_postprocessor_list = GTK_LIST_STORE(
+                gtk_builder_get_object(builder, "list-postprocessors"));
         preview = GTK_WIDGET(gtk_builder_get_object(builder, "preview"));
         main_stack = GTK_WIDGET(gtk_builder_get_object(builder, "main_stack"));
         open_last_stack =
@@ -1060,13 +1054,35 @@ activate(GtkApplication *app, gpointer data)
 
         // Setup settings
         settings = g_settings_new("org.postmarketos.Megapixels");
+        char *setting_postproc = g_settings_get_string(settings, "postprocessor");
+
+        // Initialize the postprocessing gsetting to the old processor if
+        // it was not set yet
+        if (setting_postproc == NULL || setting_postproc[0] == '\0') {
+                printf("Initializing postprocessor gsetting\n");
+                setting_postproc = malloc(512);
+                if (!mp_process_find_processor(setting_postproc)) {
+                        printf("No processor found\n");
+                        exit(1);
+                }
+                g_settings_set_string(settings, "postprocessor", setting_postproc);
+                printf("Initialized postprocessor to %s\n", setting_postproc);
+        }
+
+        // Find all postprocessors for the settings list
+        mp_process_find_all_processors(setting_postprocessor_list);
+
+        // Bind settings widgets to the actual settings
         g_settings_bind(settings,
                         "save-raw",
                         setting_dng_button,
                         "active",
                         G_SETTINGS_BIND_DEFAULT);
-
-        setting_save_dng = g_settings_get_boolean(settings, "save-raw");
+        g_settings_bind(settings,
+                        "postprocessor",
+                        setting_postprocessor_combo,
+                        "active-id",
+                        G_SETTINGS_BIND_DEFAULT);
 
         // Listen for phosh rotation
         GDBusConnection *conn =
